%%-----------------------------------------------------------------
function [fnY, fnX] = makeCoreSaS(imModel, noiseModel)

% Assuming additive white noise corruption:
%
%  noisy = s * c + n
%
% Compute Bayes least squares coring function:
% 
%  E(c | noisy) = int dc c P(c|noisy)
%               = int dc c P(noisy|c)P(c) / P(noisy)
%               = int dc Pn(noisy-c) c P(c) / int dc Pn(noisy-c)P(c)
%
% imModel and noiseModel are Nx2 matrices, containing a histogram (2nd
% column) and the corresponding bin centers.


if (size(imModel,2)~=2)
  error('imModel should be an Nx2 matrix!');
end

if (size(noiseModel,2)~=2)
  error('noiseModel should be an Nx2 matrix!');
end

imH=imModel(:,2);
imX=imModel(:,1);
delta=imX(2)-imX(1);

%figure;
%plot(imX,imH); title('Image histo');
%semilogy(imX,imH); title('Image histo');
%pause;

nH=noiseModel(:,2);
nX=noiseModel(:,1);

%figure
%hold on;plot(nX,nH,'r'); title('Noise histo');
%semilogy(nX,nH); title('Noise histo');
%pause;

%% Assume spacing is same for  noise/im histograms
if (abs(nX(2)-nX(1) - delta) > 1e-4)
  error('Image and noise histograms must have same binsize.');
end

warning off;
noisyH = conv(imH,nH);
noisyMean = conv(imX.*imH,nH); 
fnX = [imX+nX(1); imX(length(imX))+nX(2:length(nX))];
fnY = noisyMean./noisyH;

%% fnY only valid over range where image histo exists
nZeroPos = interp1(nX, [1:length(nX)]', 0,'linear');
rg = [ceil(nZeroPos):length(fnX)-(length(nX)-floor(nZeroPos))];

%rg = [length(nX):length(fnX)-length(nX)];
%rg = [1:2:2*length(imX)];
fnX=fnX(rg);
fnY=fnY(rg);

noisyH=noisyH(rg);
%figure
%hold on;plot(fnX,noisyH,'k'); title('Noisy histo');
%pause;

%%plot the bayesian estimator as compared with
%%the identity function
%figure
%hold on;
%plot(fnX,fnY,fnX,fnX,'-r');title('Bayes estimator');
%grid;axis square;
%pause;

return